'*******************************************
'*  NTSC 256 x 192 - 16 bit RGB Composite  *
'*******************************************

' P0 = YIQ

CON

_clkmode	= xinput
_xinfreq	= 60_000_000


s		= 46			'scales DAC output (s = 0..128)
r		= s * 78 / 128		'adjusts for modulator expansion

lntsc		= 3640          	'NTSC colorburst cycles per line * 16
sntsc		= 624           	'NTSC colorburst cycles per sync * 16


DAT		org
'
'	Setup
'
		setctra	#%00111		'set ctra for PLL*8
	        setfrqa	frqa_		'set frqa

		setvid	#%1110		'set vid for composite output
		setvidy	vidy_		'set yiq coefficients
		setvidi	vidi_
		setvidq	vidq_

		cfgdac0	#%11		'set bottom dac to video mode (YIQ)
		cfgpins	pinmask,pindacs	'set P0 for dac output

	reps	#256,#1
	setptra bitmap
	wrword  tffff,ptra++
	reps	#191,#2
	subptra	#2
	addptra	tfive12
	wrword  tffff,ptra
	reps	#255,#1
	nop
	wrword  tffff,--ptra
	reps	#190,#2
	nop
	subptra	tfive12
	wrword  tffff,ptra

'Spawn monitor on cog 1
		setcog	#1
		coginit monitor_pgm,monitor_ptr
'Spawn JBRenderer on cog 2
'		setcog	#2
'                coginit JBRender_pgm,JBRender_ptr

'
' Field loop --non interlaced 60Hz NTSC frame
'

field		mov	vidlp,#34	'top blanks
		call	#blank

		setptra bitmap          'Get ready to scan pixels
		setptrb bitmap

		mov     vidlp,#192	'set visible lines
line		call	#hsync		'do horizontal sync
		rdlong	bordercol,bordercolptr
		waitvid	v_bo,bordercol	'black left border
		waitvid	mode,#0		'256*9 clocks per waitvid - show screen

		reps	#128,#2		'read 256 words
		setspa	#0
		rdlongc	vidrgb,ptra++
		pusha	vidrgb

		rdlong	bordercol,bordercolptr
		waitvid	v_bo,bordercol	'black right border
    		djnz    vidlp,#line	'Do all the lines

                wrlong  one,framesync	'Set framesync

        	mov     vidlp,#28	'bottom blanks
		call	#blank

synch		mov	vidlp,#6	'high vertical syncs
		call	#vsynch
		mov	vidlp,#6	'low vertical syncs
		call	#vsyncl
		mov	vidlp,#6	'high vertical syncs
		call	#vsynch
		jmp     #field          'field loop
'
'  Subs
'
blank		call	#hsync		'blank lines
		rdlong	bordercol,bordercolptr
		waitvid	v_bl,bordercol	'black right border
		djnz	vidlp,#blank
blank_ret	ret


hsync		waitvid	v_bs,c_sh	'horizontal sync
		waitvid	v_sn,c_sl
hsync_ret	retd			'Pipeline... get ready to return
		waitvid	v_bc,c_sh
		waitvid	v_cb,c_cb
		waitvid	v_bv,c_sh 	'End of subroutine and actually return

vsynch		waitvid	v_bs,c_sh	'vertical sync high
		waitvid	v_hs,c_sl
		waitvid	v_hr,c_sh
		djnz	vidlp,#vsynch
vsynch_ret	ret


vsyncl		waitvid	v_bs,c_sh	'vertical sync low
		waitvid	v_ls,c_sl
		waitvid	v_lr,c_sh
		djnz	vidlp,#vsyncl
vsyncl_ret	ret

'
'Begin COG Data....
'

tfive12		long    512
tffff		long	$ffff

bordercol	long	$00000000
bordercolptr	long	@bordercol+$e80	'pointing to the bordercol long in hubram for other cogs to change it

bitmap		long	$08000	'start of bitmap'
framesync       long    $7ffc
one             long    1

vidlp		long	0		'Renamed x to vidlp to remove x as it may be used in another cog program
vidrgb		long	0

frqa_		long	$1E8B_A2A7		'57.272720 MHz (colorburst * 16)

vidy_		long	(110*s/128 & $FF) << 24 + ((+38*s/128) & $FF) << 16 + ((+75*s/128) & $FF) << 8 + ((+15*s/128) & $FF)
vidi_		long	(100*s/128 & $FF) << 24 + ((+76*r/128) & $FF) << 16 + ((-35*r/128) & $FF) << 8 + ((-41*r/128) & $FF)
vidq_		long	              128 << 24 + ((+27*r/128) & $FF) << 16 + ((-67*r/128) & $FF) << 8 + ((+40*r/128) & $FF)

pinmask		long	%1			'pin mask for P0
pindacs		long	%1111_101100000		'pin configuration for video DAC

		'	color base	mode		clocks/pixel	clocks/waitvid
mode		long	$00 << 24 +	$e << 20 +  9 << 13 +	256*9  'mode $D here + 9 = 256 pixels"

v_bo		long	(lntsc-sntsc-256*9)/2	'left/right border

v_bs		long	100			'before sync		horizontal sync
v_sn		long	269			'sync
v_bc		long	50			'before colorburst
v_cb		long	144			'colorburst
v_bv		long	sntsc-144-50-269-100	'before visible

v_hs		long	135			'high sync		high vertical sync
v_hr		long	lntsc/2-135-100		'high sync remainder

v_ls		long	1564			'low sync		low vertical sync
v_lr		long	lntsc/2-1564-100	'low sync remainder

v_bl		long	lntsc-sntsc		'blank visible

c_sh		long	$1_000000		'sync high
c_sl		long	$2_000000		'sync low
c_cb		long	$1_707000		'colorburst reference color

JBRender_pgm	long	@JBRenderCode+$e80
JBRender_ptr	long	0

monitor_pgm	long	$70C			'ROM entry point for monitor
monitor_ptr	long	90<<9 + 91		'Pins for one monitor


'-------------------------------------------------------------


JBRenderCode
	        org

		setspa  #0
		reps    #10,#1                  'install lookup colors into stack ram at $00
		setinda #color0
		pusha   inda++

		setpixr pixr_			'only set these once, as they don't change
		setpixg pixg_
		setpixb pixb_
		setpixa pixa_

screenloop	mov	tmpj,#511
:lp		djnz	tmpj,#:lp
		rdlong  tmpj,vgavsync wz
	if_z	jmp     #screenloop
		wrlong  zero,vgavsync

		wrlong	greenbord,borderptr

		mov	pixu_,pixu_s
		mov	pixv_,pixv_s
		mov	pixz_,pixz_s

		add     pixu_s,pixuadd		'scroll it
		and     pixu_s,pixuand

		setpix  pix_                    'set 16x32 4-bit texture w/horizontal mirroring

		setptrb mybitmap
		mov     ylp,#192
render
		setpixu pixu_                   'reset pix registers
		setpixv pixv_
		setpixz pixz_
		add	pixv_,pixvadd
		and	pixv_,pixr_		'don't allow it to mess up high word

		repd    #256,#1                 'render texture pixels
		setinda #pixels                 'ready to blend 256 pixels
		nop	#2
		nop	#2
		getpix  inda++                  'repeat getpix

'{ allow this one is for straight write
		reps    #256,#2
		setinda #pixels
		mov     tmpj,inda++
		wrword	tmpj,ptrb++		'}

{ allow this one is for when using transparencies              
		reps    #256,#3
		setinda #pixels
		mov     tmpj,inda++ wz
	if_z	rdwordc tmpj,ptrb++
	if_nz	wrword	tmpj,ptrb++		'}

		djnz    ylp,#render

		wrlong	blackbord,borderptr

'{draw white line to see how long it took, start of white line shows where raster is up to
		reps    #192,#2
		mov     myptr,mybitmap
		wrword  ffff,myptr
		add     myptr,five12		'}

'		notp	#32
'		notp	#33
'		notp	#34
'		notp	#35
'		notp	#36
'		notp	#37
'		notp	#38
'		notp	#39
'		notp	#40
'		notp	#41
'		notp	#42
'		notp	#43

		jmp     #screenloop

borderptr	long	@bordercol+$e80

greenbord	long	$00_00_80_00
blackbord	long	$00_00_00_00

vgavsync	long    $7ffc
zero		long    0
ffff		long    $ffff

ylp		long    192
mybitmap	long    $8000
tmpj		long    0

color0		long    $FF000000
color1		long    $FF000000+($d0>>3<<11)+($00>>2<<5)+($00>>3)    'solid red              propeller
color2		long    $FF000000+($80>>3<<11)+($80>>2<<5)+($00>>3)    'solid olive            propeller shaft
color3		long    $FF000000+($00>>3<<11)+($00>>2<<5)+($80>>3)    'solid dark blue        hat rim
color4		long    $FF000000+($00>>3<<11)+($a0>>2<<5)+($ff>>3)    'solid cyan             hat outside
color5		long    $FF000000+($00>>3<<11)+($78>>2<<5)+($c0>>3)    'solid cyan shadow      hat inside
color6		long    $FF000000+($FF>>3<<11)+($ff>>2<<5)+($00>>3)    'solid yellow           hat outside
color7		long    $FF000000+($C0>>3<<11)+($c0>>2<<5)+($00>>3)    'solid yellow shadow    hat inside
color8		long    $FF000000+($ff>>3<<11)+($80>>2<<5)+($00>>3)    'solid orange           hat outside
color9		long    $FF000000+($c0>>3<<11)+($60>>2<<5)+($00>>3)    'solid orange shadow    hat inside

                '       16 x 32 map, 4-bit pixels, horizontal mirroring
pix_            long    %100_101_10_0_10 << 21 + $00 << 9 + texture

pixuadd		long    $0000_1000
pixuand		long    $fffe_ffff		'to stop lower word messing high word
pixuadd2	long    $0000_8000

pixu_           long    0
pixv_           long    0
pixz_           long    0
pixr_           'long    $0000_FFFF		'same as pixa
pixg_           'long    $0000_FFFF		'same as pixa
pixb_           'long    $0000_FFFF		'same as pixa
pixa_           long    $0000_FFFF

pixvadd		long	$0000_0400

pixu_s		long    $1000_0000		'change these values to experiment
pixv_s		long    $0000_0000
pixz_s		long    $0000_0000		'try %0000_0000, $0000_8000, $0000_C000

texture         long    $00000000,$00000000	'16x32 4-bit texture, mirrored horizontally to make 32x32 texture
                long    $11110000,$00000011
                long    $11111100,$20111111
                long    $11111110,$11111111
                long    $11111110,$20001111
                long    $11111000,$20000001
                long    $00000000,$66000000
                long    $00000000,$66000000
                long    $00000000,$20000000
                long    $00000000,$44440000
                long    $00000000,$86644440
                long    $40000000,$88664444
                long    $44000000,$88666444
                long    $44400000,$88866644
                long    $44440000,$88866664
                long    $44444000,$88866664
                long    $44444000,$88866666
                long    $44444400,$88886666
                long    $64444400,$88886666
                long    $64444400,$88886666
                long    $64444440,$88886666
                long    $66444440,$88886666
                long    $66444440,$33333666
                long    $33344440,$33333333
                long    $33333440,$99993333
                long    $77533330,$99997777
                long    $77555330,$99997777
                long    $77333300,$99997777
                long    $33330000,$99993333
                long    $33000000,$33333333
                long    $00000000,$33333000
                long    $00000000,$00000000

'only used for white line
myptr		long    0
five12		long    512

pixels          'long    0[256]	'don't need this as it's in cog-ram


'=====================================
{
void
draw_quad(QUAD *q)
b	long	0
s	long	0
mtax	long	0
mtay	long	0
mxa	long	0
mtbx	long	0
mtby	long	0
mxb	long	0
mtcx	long	0
mtcy	long	0
mxc	long	0
mtdx	long	0
mtdy	long	0
mxd	long	0
qtype	long	0
yhi	long	0
qlo	long	0
t	long	0
x	long	0
y	long	0
tl	long	0
tr	long	0
xl	long	0
mxl	long	0
mxr	long	0
mmx1	long	0
mmx2	long	0
mmy1	long	0
mmy2	long	0
mxt1	long	0
mxt2	long	0
myt1	long	0
myt2	long	0
qx1	long	0
qx2	long	0
qx3	long	0
qx4	long	0
qy1	long	0
qy2	long	0
qy3	long	0
qy4	long	0
qtx1	long	0
qtx2	long	0
qtx3	long	0
qtx4	long	0
qty1	long	0
qty2	long	0
qty3	long	0
qty4	long	0

texture	long	0

	t=q->pattern;
	texture=&textures[t*4096];

	qx1=q->x1;
	qx2=q->x2;
	qx3=q->x3;
	qx4=q->x4;
	qy1=q->y1;
	qy2=q->y2;
	qy3=q->y3;
	qy4=q->y4;
	qtx1=q->tx1;
	qtx2=q->tx2;
	qtx3=q->tx3;
	qtx4=q->tx4;
	qty1=q->ty1;
	qty2=q->ty2;
	qty3=q->ty3;
	qty4=q->ty4;

	y=qy1;
	yhi=1;
	if(qy2 < y)
		yhi=2;
		y=qy2;
	if(qy3 < y)
		yhi=3;
		y=qy3;
	if(qy4 < y)
		yhi=4;
		y=qy4;
	if(yhi==2)
		  t=qx1;
		qx1=qx2;
		qx2=qx3;
		qx3=qx4;
		qx4=t;
		  t=qy1;
		qy1=qy2;
		qy2=qy3;
		qy3=qy4;
		qy4=t;
		   t=qtx1;
		qtx1=qtx2;
		qtx2=qtx3;
		qtx3=qtx4;
		qtx4=t;
		   t=qty1;
		qty1=qty2;
		qty2=qty3;
		qty3=qty4;
		qty4=t;
	if(yhi==3)
		  t=qx1;
		qx1=qx3;
		qx3=t;
		  t=qx2;
		qx2=qx4;
		qx4=t;
		  t=qy1;
		qy1=qy3;
		qy3=t;
		  t=qy2;
		qy2=qy4;
		qy4=t;
		   t=qtx1;
		qtx1=qtx3;
		qtx3=t;
		   t=qtx2;
		qtx2=qtx4;
		qtx4=t;
		   t=qty1;
		qty1=qty3;
		qty3=t;
		   t=qty2;
		qty2=qty4;
		qty4=t;
	if(yhi==4)
		  t=qx1;
		qx1=qx4;
		qx4=qx3;
		qx3=qx2;
		qx2=t;
		  t=qy1;
		qy1=qy4;
		qy4=qy3;
		qy3=qy2;
		qy2=t;
		   t=qtx1;
		qtx1=qtx4;
		qtx4=qtx3;
		qtx3=qtx2;
		qtx2=t;
		   t=qty1;
		qty1=qty4;
		qty4=qty3;
		qty3=qty2;
		qty2=t;

	qx1<<=16;
	qx2<<=16;
	qx3<<=16;
	qx4<<=16;
	qtx1<<=16;
	qtx2<<=16;
	qtx3<<=16;
	qtx4<<=16;
	qty1<<=16;
	qty2<<=16;
	qty3<<=16;
	qty4<<=16;

	qtype=0;
	qlo=qy2;
	if(qy3 > qlo)
		qlo=qy3;
		qtype=1;
	if(qy4 > qlo)
		qlo=qy4;
		qtype=2;
	t=(qy4-qy1)+1;
	if(!t) t++;
	mxa=Divide(qx4-qx1,t);
	mtax=Divide(qtx4-qtx1,t);
	mtay=Divide(qty4-qty1,t);
	t=(qy2-qy1)+1;
	if(!t) t++;
	mxb=Divide(qx2-qx1,t);
	mtbx=Divide(qtx2-qtx1,t);
	mtby=Divide(qty2-qty1,t);
	if(qtype==0)
		t=(qy3-qy4)+1;
		if(!t) t++;
		mxc=Divide(qx3-qx4,t);
		mtcx=Divide(qtx3-qtx4,t);
		mtcy=Divide(qty3-qty4,t);
		t=(qy2-qy3)+1;
		if(!t) t++;
		mxd=Divide(qx2-qx3,t);
		mtdx=Divide(qtx2-qtx3,t);
		mtdy=Divide(qty2-qty3,t);
	if(qtype==1)
		t=(qy3-qy4)+1;
		if(!t) t++;
		mxc=Divide(qx3-qx4,t);
		mtcx=Divide(qtx3-qtx4,t);
		mtcy=Divide(qty3-qty4,t);
		t=(qy3-qy2)+1;
		if(!t) t++;
		mxd=Divide(qx3-qx2,t);
		mtdx=Divide(qtx3-qtx2,t);
		mtdy=Divide(qty3-qty2,t);
	if(qtype==2)
		t=(qy3-qy2)+1;
		if(!t) t++;
		mxc=Divide(qx3-qx2,t);
		mtcx=Divide(qtx3-qtx2,t);
		mtcy=Divide(qty3-qty2,t);
		t=(qy4-qy3)+1;
		if(!t) t++;
		mxd=Divide(qx4-qx3,t);
		mtdx=Divide(qtx4-qtx3,t);
		mtdy=Divide(qty4-qty3,t);

	mxl=mxr=qx1;
	mmx1=mmx2=qtx1;
	mmy1=mmy2=qty1;
	for(;y<qlo;y++)
		if(qtype==0)
			if(y==qy4)
				mxl=qx4;
				mxa=mxc;
				mmx1=qtx4;
				mmy1=qty4;
				mtax=mtcx;
				mtay=mtcy;
			if(y==qy3)
				mxl=qx3;
				mxa=mxd;
				mmx1=qtx3;
				mmy1=qty3;
				mtax=mtdx;
				mtay=mtdy;
		if(qtype==1)
			if(y==qy4)
				mxl=qx4;
				mxa=mxc;
				mmx1=qtx4;
				mmy1=qty4;
				mtax=mtcx;
				mtay=mtcy;
			if(y==qy2)
				mxr=qx2;
				mxb=mxd;
				mmx2=qtx2;
				mmy2=qty2;
				mtbx=mtdx;
				mtby=mtdy;
		if(qtype==2)
			if(y==qy2)
				mxr=qx2;
				mxb=mxc;
				mmx2=qtx2;
				mmy2=qty2;
				mtbx=mtcx;
				mtby=mtcy;
			if(y==qy3)
				mxr=qx3;
				mxb=mxd;
				mmx2=qtx3;
				mmy2=qty3;
				mtbx=mtdx;
				mtby=mtdy;
		if(y>=240)	return;

		tl=mxl>>16;
		tr=mxr>>16;

		if(y>0)
			if(tr>=tl)
				xl=(tr-tl);
				if(xl==0) xl++;
				mxt1=mmx1;
				myt1=mmy1;
				mxt2=Divide(mmx2-mmx1,xl);
				myt2=Divide(mmy2-mmy1,xl);
			else
				t=tl;
				tl=tr;
				tr=t;
				xl=(tr-tl);
				if(xl==0) xl++;
				mxt1=mmx2;
				myt1=mmy2;
				mxt2=Divide(mmx1-mmx2,xl);
				myt2=Divide(mmy1-mmy2,xl);
			if(tl<0)
				mxt1+=(mxt2*(-tl));
				myt1+=(myt2*(-tl));
				tl=0;
			if(tr>=319) tr=319;
			if(tl<320 &&& tr>0)
				s=&(vram_ptr[(319-tl)+(320*y)]);
				for(x=tl;x<tr;x++)
					b=texture[(((myt1>>16)&63)<<6)+((mxt1>>16)&63)];
					if(b)
						s[0]=b;
					s--;
					mxt1+=mxt2;
					myt1+=myt2;
		mmx1+=mtax;
		mmy1+=mtay;
		mmx2+=mtbx;
		mmy2+=mtby;
		mxl+=mxa;
		mxr+=mxb;

}